/*===========================================================================*
 * readframe.c								     *
 *									     *
 *	procedures to read in frames					     *
 *									     *
 * EXPORTED PROCEDURES:							     *
 *	ReadFrame							     *
 *	SetFileType							     *
 *	SetFileFormat							     *
 *	JMovie2JPEG							     *
 *									     *
 *===========================================================================*/

/*
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

/*  
 *  $Header: /n/picasso/users/keving/encode/src/RCS/readframe.c,v 1.2 1993/12/22 19:19:01 keving Exp keving $
 *  $Log: readframe.c,v $
 * Revision 1.2  1993/12/22  19:19:01  keving
 * nothing
 *
 * Revision 1.1  1993/07/22  22:23:43  keving
 * nothing
 *
 */


/*==============*
 * HEADER FILES *
 *==============*/

#include <X11/Xlib.h>
#include "all.h"
#include <time.h>
#include <errno.h>
#include "mtypes.h"
#include "frames.h"
#include "prototypes.h"
#include "param.h"
#include "readframe.h"
#include "fsize.h"
#include "rgbtoycc.h"
#include "jpeg.h"
#include "../xlarc/xla2.h"

#define PPM_READ_STATE_MAGIC	0
#define PPM_READ_STATE_WIDTH	1
#define PPM_READ_STATE_HEIGHT	2
#define PPM_READ_STATE_MAXVAL	3
#define PPM_READ_STATE_DONE	4

void GetRGB(int c);
void GetFrame(int c);

/*==================*
 * STATIC VARIABLES *
 *==================*/

XColor framecols[256];
int colindex;
int xfr,yfr;
MpegFrame *theframe;

void GetRGB(int c)
{
	switch(colindex%3) {
		case 0 :
			framecols[colindex/3].red = c;
			break;
		case 1 :
			framecols[colindex/3].green = c;
			break;
		default :
			framecols[colindex/3].blue = c;
			break;
	}
	colindex ++;
}

void GetFrame(int c)
{
	if(yfr<Fsize_y) {
		if(xfr<Fsize_x*3) {
			theframe->ppm_data[yfr][xfr++] = framecols[c].red;
			theframe->ppm_data[yfr][xfr++] = framecols[c].green;
			theframe->ppm_data[yfr][xfr++] = framecols[c].blue;
		} else 
			xfr +=3;
	} else
		xfr += 3;
	if(xfr>=realWidth*3) {
		xfr = 0;
		yfr ++;
	}
	
}	

/*===========================================================================*
 *
 * ReadFrame
 *
 *	reads the given frame, performing conversion as necessary
 *	if addPath = TRUE, then must add the current path before the
 *	file name
 *
 * RETURNS:	frame modified
 *
 * SIDE EFFECTS:    none
 *
 *===========================================================================*/
void
ReadFrame(frame, fileName, conversion, addPath)
    MpegFrame *frame;
    char *fileName;
    char *conversion;
    boolean addPath;
{
	int i;
	
	for(i=0;i<256;i++)
		framecols[i].pixel = i;
	colindex = 0;
	xfr = 0;
	yfr = 0;
	
	XLAGet("colormap.col",GetRGB);	/* read the palette from the archive */
	
	Fsize_Note(frame->id,realWidth,realHeight);
	frame->rgb_maxval = 255;
	Frame_AllocPPM(frame);
	theframe = frame;
	XLAGet(fileName,GetFrame);
	PPMtoYUV(frame);
    MotionSearchPreComputation(frame);
}

