#include "xla2.h"
#include <stdio.h>
#include <string.h>

static FILE *myfile;

void usage()
{
	fprintf(stderr,"XLArc v2.1 - XLib archiving utility - FREEWARE\n");
	fprintf(stderr,"(C) 1994 - Tristan Tarrant\n");
	fprintf(stderr,"Usage :");
	fprintf(stderr,"	xlarc command archive [files]\n");
	fprintf(stderr,"Command can be any of the following :\n");
	fprintf(stderr,"	a0 - adds file to an archive with no compression\n");
	fprintf(stderr,"	a1 - adds file to an archive with LZS compression\n");
	fprintf(stderr,"	a2 - adds file to an archive with fast deflate compression\n");
	fprintf(stderr,"	a3 - adds file to an archive with slow deflate compression\n");
	fprintf(stderr,"	a4 - adds file to an archive with RLE compression\n");
	fprintf(stderr,"	v  - views files in an archive\n");
	fprintf(stderr,"	x  - extracts to disk\n");
	fprintf(stderr,"	c  - extracts to console\n");
	exit(1);
}

int fexists(char *filename)
{
	FILE *tmp;
	
	if (!(tmp=fopen(filename,"r")))
		return(0);
	else {
		fclose(tmp);
		return(1);
	}
}

int nextchar()
{
	
		return(getc(myfile));
}

void charstdout( int c )
{
	fputc(c,stdout);
}
	
void charmyfile( int c )
{
	fputc(c,myfile);
}

void main( int argc, char **argv )
{
	int r,i;
	FILE *infile;
	void *ptr;
	char *thefile;
	long dummy1, dummy2;
	int mode;
	
	
	thefile = (char *) malloc(255);
	if (argc<2) usage();
	XLA2Init();
	if (!strncmp(argv[1],"v",1)) {
		XOpenArchive(argv[2]);
		XPrintDir();
		XCloseArchive();
	} else 
	if (!strncmp(argv[1],"a1",2)) {
		fprintf(stderr,"LZS algorithm not supported yet\n");
		exit(1);
	} else
	if (!strncmp(argv[1],"a2",2)) {
		if (argc<3) usage();
		if (fexists(argv[2])) 
			XUpdateArchive(argv[2]);
		else
			XCreateArchive(argv[2]);
		for(r=3;r<argc;r++) {
			if (XLAGetFileInfo(argv[r],&dummy1,&dummy2,&mode)) {
				fprintf(stderr,"File %s already in archive\n",argv[r]);
				continue;
			} else
			if ((myfile=fopen(argv[r],"rb"))==NULL) {
				fprintf(stderr,"Could not open file %s\n",argv[r]);
			} else {
				fprintf(stderr,"Deflating %s",argv[r]);
				XLAPut(argv[r],DeflateFast,nextchar);
				fclose(myfile);
				fprintf(stderr,", done\n");
			}
		}
		XEndArchive();
	} else
	if (!strncmp(argv[1],"a3",2)) {
		if (argc<3) usage();
		if (fexists(argv[2])) 
			XUpdateArchive(argv[2]);
		else
			XCreateArchive(argv[2]);
		for(r=3;r<argc;r++) {
			if (XLAGetFileInfo(argv[r],&dummy1,&dummy2,&mode)) {
				fprintf(stderr,"File %s already in archive\n",argv[r]);
				continue;
			} else
			if ((myfile=fopen(argv[r],"rb"))==NULL) {
				fprintf(stderr,"Could not open file %s\n",argv[r]);
			} else {
				fprintf(stderr,"Deflating %s",argv[r]);
				XLAPut(argv[r],DeflateLazy,nextchar);
				fclose(myfile);
				fprintf(stderr,", done\n");
			}
		}
		XEndArchive();
	} else 
	if (!strncmp(argv[1],"a0",2)) {
		if (argc<3) usage();
		if (fexists(argv[2])) 
			XUpdateArchive(argv[2]);
		else
			XCreateArchive(argv[2]);
		for(r=3;r<argc;r++) {
			if (XLAGetFileInfo(argv[r],&dummy1,&dummy2,&mode)) {
				fprintf(stderr,"File %s already in archive\n",argv[r]);
				continue;
			} else
			if ((myfile=fopen(argv[r],"rb"))==NULL) {
				fprintf(stderr,"Could not open file %s\n",argv[r]);
			} else {
				fprintf(stderr,"Storing %s",argv[r]);
				XLAPut(argv[r],None,nextchar);
				fclose(myfile);
				fprintf(stderr,", done\n");
			}
		}
		XEndArchive();
	} else 
	if (!strncmp(argv[1],"a4",2)) {
		if (argc<3) usage();
		if (fexists(argv[2])) 
			XUpdateArchive(argv[2]);
		else
			XCreateArchive(argv[2]);
		for(r=3;r<argc;r++) {
			if (XLAGetFileInfo(argv[r],&dummy1,&dummy2,&mode)) {
				fprintf(stderr,"File %s already in archive\n",argv[r]);
				continue;
			} else
			if ((myfile=fopen(argv[r],"rb"))==NULL) {
				fprintf(stderr,"Could not open file %s\n",argv[r]);
			} else {
				fprintf(stderr,"RLEing %s",argv[r]);
				XLAPut(argv[r],RLE,nextchar);
				fclose(myfile);
				fprintf(stderr,", done\n");
			}
		}
		XEndArchive();
	} else 
	if (!strncmp(argv[1],"c",1)) {
		if (argc<3) usage();
		XOpenArchive(argv[2]);
		for(r=3;r<argc;r++) {
			fprintf(stderr,"Extracting %s\n",argv[r]);
			XLAGet(argv[r],charstdout);
			fflush(stdout);
		}
		XCloseArchive();
	} else
	if (!strncmp(argv[1],"x",1)) {
		if (argc<3) usage();
		XOpenArchive(argv[2]);
		for(r=3;r<argc;r++) {
			i = XLAFindFirst(argv[r], thefile);
			while (i) {
				fprintf(stderr,"Extracting %s\n",thefile);
				myfile=fopen(thefile,"wb");
				XLAGet(thefile,charmyfile);
				fclose(myfile);
				i = XLAFindNext(thefile);
			}
		}
		XCloseArchive();
	} else	usage();
}
