
#include <stdio.h>
#include <string.h>
#include <ncurses/curses.h>
#include "xla2.h"
static FILE *myfile;
static FILE *thepipe;
static char AFileName[80];
static char ArchiveDir[255][80];
int maxlen, filewidth;

void usage()
{
	endwin();
	fprintf(stderr,"XLView v2.1 - XLArc viewing utility - FREEWARE\n");
	fprintf(stderr,"(C) 1994 - Tristan Tarrant\n");
	fprintf(stderr,"Usage :");
	fprintf(stderr,"	xlview archive\n");
	exit(1);
}

int fexists(char *filename)
{
	FILE *tmp;
	
	if (!(tmp=fopen(filename,"r")))
		return(0);
	else {
		fclose(tmp);
		return(1);
	}
}

int nextchar()
{
	return(getc(myfile));
}

void charpipeout( int c )
{
	putc( c, thepipe );
}
	
int readfiles()
{
	int r,i;

	i = 0;
	r = XLAFirst(AFileName);
	while (r) {
		if (strlen(AFileName)>maxlen) maxlen=strlen(AFileName);
		strcpy(ArchiveDir[i++],AFileName);
		r = XLANext(AFileName);
	}
	maxlen++;
	filewidth = (int) COLS / maxlen; 
	return(i);
}	

void DisplayFiles(int select, int nf)
{
	int index;
	
	for(index=0;index<nf;index++) {
		if (index==select) attron(A_REVERSE);
		mvprintw(2+(index / filewidth),maxlen*(index % filewidth),
							"%s",ArchiveDir[index]);
		if (index==select) attroff(A_REVERSE);
	}
}

void DisplayInfoWin(int sel)
{
	int infow, infoh, mode;
	WINDOW *infowin;
	long origsize, compsize;
	
	infow = maxlen+2;
	if (infow<27) infow=27;
	infowin = newwin(7,infow,(LINES-7)/2,(COLS-infow)/2);
	leaveok(infowin,TRUE);
	box(infowin,0,0);
	wattron(infowin,A_REVERSE);
	mvwprintw(infowin,1,1,"     File Information    ");
	wattroff(infowin,A_REVERSE);
	mvwprintw(infowin,2,1,"%s",ArchiveDir[sel]);
	XLAGetFileInfo(ArchiveDir[sel],&origsize,&compsize,&mode);
	mvwprintw(infowin,3,1,"Original size   : %7d",origsize);
	mvwprintw(infowin,4,1,"Compressed size : %7d",compsize);
	switch (mode) {
		case None :
			mvwprintw(infowin,5,1,"Mode used       : None");
			break;
		case LZS :
			mvwprintw(infowin,5,1,"Mode used       : LZS77");
			break;
		case DeflateFast :
			mvwprintw(infowin,5,1,"Mode used       : Deflat1");
			break;
		case DeflateLazy :
			mvwprintw(infowin,5,1,"Mode used       : Deflat9");
			break;
		case RLE :
			mvwprintw(infowin,5,1,"Mode used       :     RLE");
			break;
		default:
			mvwprintw(infowin,5,1,"Mode used       : Unknown");
			
	}
	
	wrefresh(infowin);
	getch();
	delwin(infowin);
}

void DisplayHelpWin()
{
	WINDOW *helpwin;
	
	helpwin = newwin(11,48,(LINES-15)/2,(COLS-48)/2);
	leaveok(helpwin,TRUE);
	box(helpwin,0,0);
	wattron(helpwin,A_REVERSE);
	mvwprintw(helpwin,1,1,"    XLView v2.1 - (C) 1994 Tristan Tarrant    ");
	wattroff(helpwin,A_REVERSE);
	mvwprintw(helpwin,3,1,"h,H,F1      - This help page");
	mvwprintw(helpwin,4,1,"Cursor keys - Move selection");
	mvwprintw(helpwin,5,1,"i,I         - Display file information");
	mvwprintw(helpwin,6,1,"x,X         - Extract file to disk");
	mvwprintw(helpwin,7,1,"v,V,Return,");
	mvwprintw(helpwin,8,1,"Spacebar    - View file using pager");
	mvwprintw(helpwin,9,1,"q,Q         - Quit XLView");
	wrefresh(helpwin);
	getch();
	delwin(helpwin);
}	

void DisplayStatus(char *filename)
{
	attron(A_REVERSE);
	mvprintw(0,0," XLView v2.1 - F1 or H for help - Viewing %-37s",filename);
	attroff(A_REVERSE);
}

void DisplayFileWithPager(int sel)
{
	endwin();
	thepipe = popen("less","w");
  XLAGet(ArchiveDir[sel],charpipeout);
  pclose(thepipe);
  initscr();
	cbreak();
	noecho();
	keypad(stdscr,TRUE);
	leaveok(stdscr,TRUE);
	erase();
	touchwin(stdscr);
}

void main( int argc, char **argv )
{
	int i, nfiles, selected, viewing;
	FILE *infile;
	void *ptr;
	char *thefile;
	
	initscr();
	cbreak();
	noecho();
	keypad(stdscr,TRUE);
	leaveok(stdscr,TRUE);
	
	thefile = (char *) malloc(255);
	if (argc!=2) usage();
	
	XLA2Init();
	
	maxlen = 0;
	selected = 0;
	viewing = 1;
	
	if (!XOpenArchive(argv[1])) usage();
	else {
		nfiles = readfiles();
		while (viewing) {
			erase();
			DisplayStatus(argv[1]);
			DisplayFiles( selected, nfiles );
			refresh();
			switch (i=getch()) {
				case KEY_LEFT:
					if (selected>0) selected--;
					break;
				case KEY_RIGHT:
					if (selected<nfiles-1) selected++;
					break;
				case KEY_UP:
					if (selected>=filewidth) selected -= filewidth;
					break;
				case KEY_DOWN:
					if (selected<nfiles-filewidth) selected += filewidth;
					break;
				case 'Q': case 'q':
					viewing = 0;
					break;
				case 'I': case 'i':
					DisplayInfoWin(selected);
					break;
				case 'H': case 'h': case KEY_F(1):
					DisplayHelpWin();
					break;
				case 10 : case 'V': case 'v': case ' ':
					DisplayFileWithPager(selected);
					break;
				default:
					beep();
			}
		}
		XCloseArchive();
	}
	endwin();
}
