#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <stdio.h>
#include "xmgargs.h"
#include "xmgwin.h"
#include "xmgdisplay.h"

#define BARWIDTH 360
Window XMGWindow,XMGGrab,XMGTmpInput,XMGOutInput;
GC gc;
Colormap cmap;
XFontStruct *font;
int CGray1, CGray2, CGray3, CBlack, CWhite, CRed, CGreen;
int GrabPercent, EncodePercent;

void LoadFont()
{
	char *fontname = "9x15";
	
	if((font = XLoadQueryFont(display,fontname)) == NULL) {
		fprintf(stderr,"Cannot open 9x15 font\n");
		exit(1);
	}
	XSetFont(display,gc,font->fid);
}

int GetColor(char *colorname)
{
	XColor colordef;
	
	XParseColor(display,cmap,colorname,&colordef);
	XAllocColor(display,cmap,&colordef);
	return(colordef.pixel);
}

void InitWindow()
{
	XEvent event;
	XTextProperty TitleProp,IconProp;
	XSizeHints hints;
	char *Title = "X MPEG Grabber v1.0";
	char *Icon  = "XMG v1.0";
	XSetWindowAttributes attributes;
	Cursor cursor;
	
	cmap = DefaultColormap(display,screen);
	CGray1 = GetColor("gray50");
	CGray2 = GetColor("gray70");
	CGray3 = GetColor("gray90");
	CBlack = GetColor("black");
	CWhite = GetColor("white");
	CRed   = GetColor("red");
	CGreen = GetColor("green");
	XMGWindow = XCreateSimpleWindow(display,RootWindow(display,screen),0,0,
		BARWIDTH+18,200,0, BlackPixel(display,screen), CGray2);
	XMGGrab = XCreateSimpleWindow(display,XMGWindow,145,176,60,17,0,
		BlackPixel(display,screen), CGray2);
	XMGOutInput = XCreateSimpleWindow(display,XMGWindow,8,114,BARWIDTH,17,0,
		BlackPixel(display,screen), CGray2);
	XMGTmpInput = XCreateSimpleWindow(display,XMGWindow,8,153,BARWIDTH,17,0,
	    BlackPixel(display,screen), CGray2);
	cursor = XCreateFontCursor(display,XC_xterm);
	XDefineCursor(display,XMGTmpInput,cursor);
	XDefineCursor(display,XMGOutInput,cursor);
	cursor = XCreateFontCursor(display,XC_hand1);
	XDefineCursor(display,XMGGrab,cursor);
	XSelectInput(display,XMGWindow,ExposureMask);
	XSelectInput(display,XMGGrab,ButtonPressMask|ButtonReleaseMask);
	XSelectInput(display,XMGTmpInput,KeyPressMask|EnterWindowMask|
		LeaveWindowMask);
	XSelectInput(display,XMGOutInput,KeyPressMask|EnterWindowMask|
		LeaveWindowMask);
	XStringListToTextProperty(&Title,1,&TitleProp);
	XSetWMName(display,XMGWindow,&TitleProp);
	XStringListToTextProperty(&Icon,1,&IconProp);
	XSetWMIconName(display,XMGWindow,&IconProp);
	hints.x = 0;
	hints.y = 0;
	hints.min_width = BARWIDTH+18;
	hints.max_width = BARWIDTH+18;
	hints.min_height = 200;
	hints.max_height = 200;
	hints.width_inc = 0;
	hints.height_inc = 0;
	hints.flags = PSize | PMinSize | PMaxSize | PResizeInc;
	XSetWMNormalHints(display,XMGWindow,&hints);
	XMapWindow(display,XMGTmpInput);
	XMapWindow(display,XMGOutInput);
	XMapWindow(display,XMGGrab);
	XMapWindow(display,XMGWindow);
	GrabPercent = 0;
	EncodePercent = 0;
	gc = XCreateGC(display,XMGWindow,0,NULL);
	XSetForeground(display,gc,CBlack);
	XSetFillStyle(display,gc,FillSolid);
	LoadFont();
	while(1) {
		XNextEvent(display,&event);
		if(event.type==Expose)
			break;
	}	
	DrawWindow(1);		
}

void DrawBox(Drawable d, int x1, int y1, int x2, int y2, int raised)
{
	XSetForeground(display,gc,CGray2);
	XFillRectangle(display,d,gc,x1,y1,x2-x1+1,y2-y1+1);
	if(raised)
		XSetForeground(display,gc,CGray3);
	else
		XSetForeground(display,gc,CGray1);
	XDrawLine(display,d,gc,x1,y1,x2,y1);
	XDrawLine(display,d,gc,x1,y1,x1,y2);
	if(raised)
		XSetForeground(display,gc,CGray1);
	else
		XSetForeground(display,gc,CGray3);
	XDrawLine(display,d,gc,x1,y2,x2,y2);
	XDrawLine(display,d,gc,x2,y1,x2,y2);
}

void DrawRectangle(Drawable d, int x1, int y1, int x2, int y2, int raised)
{
	if(raised)
		XSetForeground(display,gc,CGray3);
	else
		XSetForeground(display,gc,CGray1);
	XDrawLine(display,d,gc,x1,y1,x2,y1);
	XDrawLine(display,d,gc,x1,y1,x1,y2);
	if(raised)
		XSetForeground(display,gc,CGray1);
	else
		XSetForeground(display,gc,CGray3);
	XDrawLine(display,d,gc,x1,y2,x2,y2);
	XDrawLine(display,d,gc,x2,y1,x2,y2);
}

void PrintString(Drawable d,int x,int y,char *buffer,int first, int max)
{
	int numchar;
	
	if ((numchar=max)+first>strlen(buffer))
		numchar = strlen(buffer)-first;
	XDrawString(display,d,gc,x,y,buffer+first,numchar);
}

void DrawWindow(int WholeRedraw)
{
	XEvent event;
	
	if(!WholeRedraw) {
		if(XCheckWindowEvent(display,XMGWindow,ExposureMask,&event))
			WholeRedraw = 1;
	}
	
	if(WholeRedraw) {
		DrawBox(XMGWindow,8,30,BARWIDTH+9,45,False);
		DrawBox(XMGWindow,8,72,BARWIDTH+9,87,False);
		DrawBox(XMGOutInput,0,0,BARWIDTH-1,16,True);
		DrawBox(XMGTmpInput,0,0,BARWIDTH-1,16,True);
		DrawBox(XMGGrab,0,0,59,16,True);
		XSetForeground(display,gc,CBlack);
		XDrawString(display,XMGWindow,gc,10,22,"Grabbing :",10);
		XDrawString(display,XMGWindow,gc,10,64,"Encoding :",10);
		XDrawString(display,XMGWindow,gc,8,109,"Output File :",13);
		XDrawString(display,XMGOutInput,gc,2,13,OutputFile,strlen(OutputFile));
		XDrawString(display,XMGWindow,gc,8,148,"Temp File   :",13);
		XDrawString(display,XMGTmpInput,gc,2,13,TmpFile,strlen(TmpFile));
		XDrawString(display,XMGGrab,gc,12,14,"Grab",4);
	}
	XSetForeground(display,gc,CGreen);
	XFillRectangle(display,XMGWindow,gc,9,31,(GrabPercent*BARWIDTH)/100,14);
	XSetForeground(display,gc,CRed);
	XFillRectangle(display,XMGWindow,gc,9,73,(EncodePercent*BARWIDTH)/100,14);
	XFlush(display);
}

void DrawCursor(Drawable d, char *buffer, int Show)
{
	if(Show)
		XSetForeground(display,gc,CBlack);
	else 
		XSetForeground(display,gc,CGray2);
		
	XFillRectangle(display,d,gc,XTextWidth(font,buffer,strlen(buffer))+2,
		1,8,15);
}

void WaitForButtonPress()
{
	XEvent event;
	int quit=0;
	int pressed=0;
	char *buffer;
	char charbuf[10];
	int bufsize = 10;
	KeySym keysym;
	XComposeStatus compose;
	int count,length;
	
	while(!quit) {
		XNextEvent(display,&event);
		switch(event.type) {
			case Expose :
				if (event.xexpose.count == 0)
					DrawWindow(1);
				break;
			case ButtonPress :
				if((event.xbutton.window==XMGGrab)&&(!pressed)) {
					DrawRectangle(XMGGrab,0,0,59,16,False);
					pressed = 1;
				}
				break;
			case ButtonRelease :
				if((event.xbutton.window==XMGGrab)&&(pressed)) {
					DrawRectangle(XMGGrab,0,0,59,16,True);
					XFlush(display);
					pressed = 0;
					quit = 1;
				}
				break;
			case EnterNotify :
				if(event.xcrossing.window==XMGOutInput)
					DrawCursor(XMGOutInput,OutputFile,True);
				else
					DrawCursor(XMGTmpInput,TmpFile,True);
				break;
			case LeaveNotify :
				if(event.xcrossing.window==XMGOutInput)
					DrawCursor(XMGOutInput,OutputFile,False);
				else
					DrawCursor(XMGTmpInput,TmpFile,False);
				break;
			case KeyPress :
				if((event.xkey.window==XMGOutInput) ||
				   (event.xkey.window==XMGTmpInput)) {
					if(event.xkey.window==XMGOutInput)
						buffer = OutputFile;
					else
						buffer = TmpFile;
					count = XLookupString(&event.xkey,charbuf,bufsize,&keysym,
						&compose);
					charbuf[count] = 0;
					if(((keysym >= XK_KP_Space) &&
						(keysym <= XK_KP_9)) ||
					   ((keysym>= XK_space) &&
						(keysym<= XK_asciitilde))) {
							if ((strlen(buffer)+strlen(charbuf))>(BARWIDTH/9))
								XBell(display,100);
							else {
								DrawCursor(event.xkey.window,buffer,False);
								strcat(buffer,charbuf);
							}
					} else
					if((keysym >= XK_Shift_L) &&
					   (keysym <= XK_Hyper_R)) ; /* Do nothing */
					else
					if((keysym==XK_BackSpace) ||
					   (keysym==XK_Delete)) {
						if((length=strlen(buffer))>0) {
							buffer[length-1] = 0;
							DrawBox(event.xkey.window,0,0,BARWIDTH-1,16,True);
						} else
							XBell(display,100);
					} else
						XBell(display,100);
					XSetForeground(display,gc,CBlack);
					XDrawString(display,event.xkey.window,gc,2,13,buffer,
						strlen(buffer));
					DrawCursor(event.xkey.window,buffer,True);
				}
				break;
			default :
				break;
		}
	}	
}

void CloseWindow()
{
	XUnloadFont(display,font->fid);
	XFreeGC(display,gc);
}
