#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xos.h>
#ifdef MITSHM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#endif
#include <sys/time.h>
#include <stdio.h>
#include "xmgdisplay.h"
#include "xmgwin.h"
#include "xmgargs.h"
#include "encode/headers/param.h"
#include "xlarc/xla2.h"
	
XImage *frame;					/* Data structure that will contain a frame */
int framesize;
int byteinframe,colinframe;
XColor colors[256];

int NextPixel()	/* Function to output the frame's pixels */
{
	if(byteinframe==framesize)
		return(-1);
	else 
		return((int)*(frame->data+byteinframe++));
}
int NextRGB()
{
	int colindex;
	
	if(colinframe==768)
		return(-1);
	else {
		switch(colinframe%3) {
			case 0 :
				colindex = colors[colinframe/3].red;
				break;
			case 1 :
				colindex = colors[colinframe/3].green;
				break;
			default :
				colindex = colors[colinframe/3].blue;
				break;
		}
		colinframe ++;
		return(colindex >> 8);
	}
}



void DoGrab(Display *display, Window w, int Microseconds, int Frames, 
		char *fname)
{
	struct timeval time1, time2;	/* Beginning and end of wait loop */
	struct timezone tz;				/* Used by gettimeofday */
	int nframes;					/* Number of captured frames */
	int winwidth, winheight, windepth, dummy;
	Window root;
	Visual *visual;
	int i,status;
	int aframe;
	char framename[32];
	XWindowAttributes windowattrs;
	Window tstWindow;
	GC gc;
	
#ifdef MITSHM
	XShmSegmentInfo shminfo;
#endif
	
	gc = XDefaultGC(display,screen);
	nframes = 0;
	XGetGeometry(display,w,&root,&dummy,&dummy,&winwidth,&winheight,&dummy,
		&windepth);					/* Find size of grab window */
	yuvWidth = realWidth = winwidth;
	yuvHeight = realHeight = winheight;
	
	visual = XDefaultVisual(display,screen);
	XCreateArchive(fname);				/* Initialise the frame archive */

	/* Grab the colormap for the specified window */
	XGetWindowAttributes(display,w,&windowattrs);
	for(i=0;i<256;i++)
		colors[i].pixel = i;
	XQueryColors(display,windowattrs.colormap,colors,256);
	colinframe = 0;	/* Start writing colormap from beginning */
	XLAPut("colormap.col",None,NextRGB); /* Write colormap to archive */
#ifdef MITSHM
	if(UseMITSHM) {	
		frame = XShmCreateImage(display,visual,windepth,ZPixmap,NULL,&shminfo,
			winwidth,winheight);	/* Create shared memory XImage */
		framesize = frame->bytes_per_line*frame->height;
		yuvWidth = realWidth = frame->bytes_per_line;
		/* Set up IPC SHM parameters */
		shminfo.shmid = shmget(IPC_PRIVATE,framesize,IPC_CREAT|0777);
		shminfo.shmaddr = frame->data = shmat(shminfo.shmid,0,0);
		shminfo.readOnly = False;
		XSync(display,False);
		status = XShmAttach(display,&shminfo);	/* Attach shared segment */
		XSync(display,False);
		if(!status) {	/* Someone lied : SHM doesn't work so disable it ! */
			fprintf(stderr,"MITSHM disabled\n");
			UseMITSHM = False;
		}
	}
#endif
	if(!UseMITSHM) {
		frame = XGetImage(display,w,0,0,winwidth,winheight,AllPlanes,ZPixmap);
		framesize = frame->bytes_per_line*frame->height;
		yuvWidth = realWidth = frame->bytes_per_line;
		XDestroyImage(frame);
	}
	for(aframe=0;aframe<Frames;aframe++) {
		XGrabServer(display);			/* Take control of the server */
#ifdef MITSHM
		if(UseMITSHM)
			XShmGetImage(display,w,frame,0,0,AllPlanes);
#endif
		if(!UseMITSHM)
			frame = XGetImage(display,w,0,0,winwidth,winheight,AllPlanes,\
				ZPixmap);
		XSync(display,False);
		byteinframe = 0;
		sprintf(framename,"frame%d.raw",aframe);
		XLAPut(framename,RLE,NextPixel);	/* Write frame to archive */
		if(!UseMITSHM)
			XDestroyImage(frame);		/* Destroy the frame */
		if(Verbose) {
			GrabPercent = aframe*100/Frames;
			DrawWindow(0);
		}
		XUngrabServer(display);			/* Release the server */
		XSync(display,False);
		gettimeofday(&time1,&tz);		/* Get time */
		time1.tv_usec += Microseconds;
		while(time1.tv_usec>1000000) {		/* Set end of wait loop */
			time1.tv_sec++;
			time1.tv_usec -= 1000000;
		} 
		do {							/* Wait for Microseconds */
			gettimeofday(&time2,&tz);
		} while ((time1.tv_sec>time2.tv_sec)||\
			((time1.tv_sec==time2.tv_sec)&&(time1.tv_usec>time2.tv_usec)));	
	}
	XEndArchive();						/* Close the frame archive */
	if(Verbose) {
		GrabPercent = aframe*100/Frames;
		DrawWindow(0);
	}
#ifdef MITSHM
	if(UseMITSHM) {						/* Clean up the Shared memory bits */
		XShmDetach(display,&shminfo);
		XDestroyImage(frame);
		shmdt(shminfo.shmaddr);
		shmctl(shminfo.shmid,IPC_RMID,0);
	}
#endif
}
	