/* 
 * Program : X MPEG Grabber - xmg
 * Author  : Tristan Tarrant (tristant@cogs.susx.ac.uk)
 * Module  : xmgmain.c
 * Purpose : Main initialisaation stuff, command line parsing, etc. 
 *
 */

/* Include standard X11 libraries */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xos.h>
#include <X11/extensions/shape.h>
#ifdef MITSHM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#endif /* MITSHM */
#include <X11/Xmu/WinUtil.h>
#include <stdio.h>
#include <string.h>

/* Include custom stuff */ 
#include "xmgwin.h"
#include "xmgerr.h"
#include "xmgargs.h"
#include "xmggrab.h"
#include "xmgdisplay.h"
#include "all.h"
#include "param.h"
#include "mpeg.h"

/* Defines */

#define PROGNAME "XMG - X Mpeg Grabber"
#define VERMAJ   1
#define VERMIN   0

/* Global variables */
Display *display;
int screen;
char program_name[1024];
Window XMGWin;

/*
 * Display the program's banner
 */
 
void banner()
{
	fprintf(stderr,PROGNAME);
	fprintf(stderr," v%d.%d\n",VERMAJ,VERMIN);
	fprintf(stderr,"Copyright (C) Tristan Tarrant 1994-5\n");
	fprintf(stderr,"Portions (C) The Regents Of The University of California\n");
}
 
/*
 * Report the syntax for calling xmg:
 */
void usage()
{
    fprintf(stderr,
    	"usage:  %s [-options ...]\n\n", program_name);
    fprintf(stderr,
    	"where options include:\n");
    fprintf(stderr,
    	"    -help             Print this message\n");
    fprintf(stderr,
    	"    -display host:dpy X server to contact\n");
    fprintf(stderr,
    	"    -frames nframes   Number of total frames to grab\n");
    fprintf(stderr,
    	"    -fps nfps         Number of frames/sec to grab\n");
    fprintf(stderr,
    	"    -fpm nfpm         Number of frames/min to grab (for slow anims)\n");
    fprintf(stderr,
    	"    -terse            Do not use X interface\n");
    fprintf(stderr,
    	"    -output filename  Specify output file (default: xmgout.mpg)\n");
    fprintf(stderr,
    	"    -tmp filename     Specify temporary file (default: xmgtmp.$$$)\n");
    fprintf(stderr,
    	"    -param filename   Specify paramter file for MPEG encoding.\n");
    
    exit (1);
}

void main(int argc, char **argv)
{
	register int i;
	int tree = 0, stats = 0, bits = 0, events = 0, wm = 0, size  = 0, shape = 0;
	int frame = 0, children = 0;
	Window window;
	FILE *ofp;
#ifdef MITSHM
	int shmvermaj, shmvermin, shmpix;
#endif

	banner();	
	strcpy(program_name,argv[0]); 
	parseargs(argc,argv);
	if((ofp = fopen(ParamFile,"rb"))==NULL) {
		writeparamfile();
		strcpy(ParamFile,"xmg.param");
	} else
		fclose(ofp);
	ReadParamFile(ParamFile,ENCODE_FRAMES);
	if(Help)
		usage();
	
	if ((display = XOpenDisplay(DisplayName))==NULL) {
		fprintf(stderr,"Can't open display %s.\n",DisplayName);
		exit(1);
	}
	screen = DefaultScreen(display);
	
#ifdef MITSHM
	UseMITSHM = XShmQueryExtension(display);
	if(UseMITSHM&&Verbose) {
		fprintf(stderr,"MIT Shared Memory available : ");
		XShmQueryVersion(display,&shmvermaj,&shmvermin,&shmpix);
		fprintf(stderr,"MITSHM v%d.%d",shmvermaj,shmvermin);
		if(shmpix)
			fprintf(stderr," (can do pixmaps)\n");
		else
			fprintf(stderr,"\n");
	}
#endif
	if(Verbose) {
		InitWindow();
		WaitForButtonPress();
	} else 
		fprintf(stderr,"\nPlease select the window to grab with the mouse.\n");
	window = Select_Window(display);
	if (window && !frame) {
		Window root;
	    int dummyi;
	    unsigned int dummy;

		if (XGetGeometry (display, window, &root, &dummyi, &dummyi,
			&dummy, &dummy, &dummy, &dummy) && window != root)
	      		window = XmuClientWindow (display, window);
	}
	DoGrab(display,window,Delay,Frames,TmpFile);
	strcpy(outputFileName,OutputFile);
	Frame_Init();
	if ((ofp=fopen(outputFileName,"w"))==NULL) {
		fprintf(stderr,"ERROR : Could not open output file\n");
		exit(1);
	}
	XOpenArchive(TmpFile);
	GenMPEGStream(-1, -1, Frames-1, customQtable, customNIQtable,
		Frames,ofp,outputFileName);
	XCloseArchive();
	Frame_Exit();
	remove(TmpFile);
	if(Verbose)
		CloseWindow();
	XCloseDisplay(display);
	exit(0);
}
